using System;
using System.Drawing;
using System.Windows.Forms;

namespace HexaGame
{
    public partial class Form1 : Form
    {
        private const int gridSize = 20;
        private int[,] map = new int[gridSize, gridSize]; //  
        private int playerX = 10; //   X ǥ
        private int playerY = 0;  //   Y ǥ
        private System.Windows.Forms.Timer gravityTimer;
        private System.Windows.Forms.Timer moveTimer;
        private bool movingLeft = false;
        private bool movingRight = false;
        
        public Form1()
        {
            InitializeComponent();
            this.ClientSize = new Size(400, 400);
            this.DoubleBuffered = true;

            InitMap();
            InitTimers();
        }

        private void InitMap()
        {
            //  ʱȭ (谪 -1,  0)
            for (int y = 0; y < gridSize; y++)
            {
                for (int x = 0; x < gridSize; x++)
                {
                    if (x == 0 || x == gridSize - 1 || y == gridSize - 1)
                        map[y, x] = -1; //  
                    else
                        map[y, x] = 0;
                }
            }
            map[playerY, playerX] = new Random().Next(1, 11); //   
        }

        private void InitTimers()
        {
            gravityTimer = new System.Windows.Forms.Timer { Interval = 1000 }; // 1ʸ ߷ 
            gravityTimer.Tick += GravityMove;
            gravityTimer.Start();

            moveTimer = new System.Windows.Forms.Timer { Interval = 100 }; // ε巯 ̵ ó
            moveTimer.Tick += MovePlayer;
        }

        private void GravityMove(object sender, EventArgs e)
        {
            if (playerY < gridSize - 2 && map[playerY + 1, playerX] == 0)
            {
                map[playerY, playerX] = 0;
                playerY++;
                map[playerY, playerX] = new Random().Next(1, 11);
                Invalidate();
            }
        }

        protected override void OnKeyDown(KeyEventArgs e)
        {
            if (e.KeyCode == Keys.Left)
            {
                movingLeft = true;
                moveTimer.Start();
            }
            else if (e.KeyCode == Keys.Right)
            {
                movingRight = true;
                moveTimer.Start();
            }
        }

        protected override void OnKeyUp(KeyEventArgs e)
        {
            if (e.KeyCode == Keys.Left) movingLeft = false;
            if (e.KeyCode == Keys.Right) movingRight = false;
            
            if (!movingLeft && !movingRight)
                moveTimer.Stop();
        }

        private void MovePlayer(object sender, EventArgs e)
        {
            if (movingLeft && playerX > 1 && map[playerY, playerX - 1] == 0)
            {
                map[playerY, playerX] = 0;
                playerX--;
                map[playerY, playerX] = new Random().Next(1, 11);
            }
            else if (movingRight && playerX < gridSize - 2 && map[playerY, playerX + 1] == 0)
            {
                map[playerY, playerX] = 0;
                playerX++;
                map[playerY, playerX] = new Random().Next(1, 11);
            }
            Invalidate();
        }

        protected override void OnPaint(PaintEventArgs e)
        {
            base.OnPaint(e);
            int cellSize = 20;
            for (int y = 0; y < gridSize; y++)
            {
                for (int x = 0; x < gridSize; x++)
                {
                    if (map[y, x] == -1)
                        e.Graphics.FillRectangle(Brushes.Black, x * cellSize, y * cellSize, cellSize, cellSize);
                    else if (map[y, x] > 0)
                        e.Graphics.FillRectangle(new SolidBrush(Color.FromArgb(255, map[y, x] * 20, 100, 200)), x * cellSize, y * cellSize, cellSize, cellSize);
                    else
                        e.Graphics.DrawRectangle(Pens.Gray, x * cellSize, y * cellSize, cellSize, cellSize);
                }
            }
        }
    }
}
